%Model inputs
Q10 = 150; 
Q20 = 400; 
P1 = 3; 
P2i = 8;
eta1 = 20; 
eta2 = 15; 
gamma = 5;
v = 0.5;
I = 200;
r = 0.05; 

%Probability inputs
lambda = 2;
a = 1.5; 
Ti = 0.4;
u = 0.2; 

%Grid size inputs
n = 1500; 
k = 500; 

%end values on grid
Tmax = 7; 
Q1max = 70; 

%Calculate step length
dQ1 = (Q1max-eta1*P1)/n ;
dT = Tmax/k ; %Check that you get integer when dividing u by this
dum = round(u/dT);
dT = u/dum;
Tmax = k*dT;

%Set up grid
Q1 = zeros (1,n);
T = zeros (1,k);
Q1(1) = eta1*P1;
for i = 2:length(Q1)
    Q1(i) = Q1(i-1) + dQ1;
end
T(1) = 0;
for i = 2: length (T)
    T(i) = T(i-1) + dT;
end

%Set up matrices for monopoly value
Vm = zeros (n,k);


%Calculate monopoly values
T_s_m = ones(1,n)*(Tmax-2*dT);
Vm(1, k) =   V2m(Tmax, eta2, r, Q20)-I;
dum = 0;
Q1Inf = n;
for j = k-1 : -1: k - u/dT
    Vm(1, j) = max((V2m(T(j), eta2, r, Q20)-I), (lambda* Vm(1,k))/(lambda + r) );
    if (Vm(1 , j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0
         T_s_m(1) = T(j);
         dum = 1;
    end
end

for j = k- u/dT -1 : -1: 1
    Vm(1, j) = max((V2m(T(j), eta2, r, Q20)-I), (lambda* Vm(1,j + u/dT))/(lambda + r) );
    if (Vm(1 , j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0
         T_s_m(1) = T(j);
         dum = 1;
    end
end

for i = 2 :n
    Vm(i, k) =   V2m(Tmax, eta2, r, Q20)-I;
    dum = 0;
    for j = k-1: -1 : k-u/dT
                Vm(i, j) = max(V2m(T(j), eta2, r, Q20)-I,  OVm(P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,k)));
        if (Vm(i, j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0 && dum == 0 
            dum = 1;
            T_s_m(i) = T(j);
        end
    end
        
    
    for j = k-u/dT-1 : -1 : 1  
        Vm(i, j) = max(V2m(T(j), eta2, r, Q20)-I,  OVm (P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,j+ u/dT)));
        if (Vm(i, j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0
            dum = 1;
            T_s_m(i) = T(j);
        end
    end
    
    if abs(T_s_m(i)- (Tmax-2*dT)) < 0.00001 % double, so cannot compare exact values
        Q1Inf = i-1;
        break;
    end
end

for i = Q1Inf: n
    Vm(i,k) = OV2m (P1, Q1(i), eta1, dQ1, Vm(i-1,k), r);
    for j = k-1: -1: k-u/dT
        Vm(i,j) = OVm(P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,k));
    end
    for j = k-u/dT - 1: -1: 1
        Vm(i,j) = OVm(P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,j+u/dT));
    end
end

%Calculate duopoly values

%Initial values at Q1 = eta1*P1. No prob of leader failing, and no gain
%from selling more old products.
Vd = zeros (n,k); 
F = zeros(n,k);
T_s_d = zeros(1,n);
P2e = zeros(1,n);

%Assume investment at thetamax
pdi = 0; % prob leader failing = 0 for Q1 = eta1*P1, as time has gone to inf.
P2e(1) = P2eOPT(v, Q20, gamma, eta2, P2i, Ti, Q1(1), eta1, P1, a, Q10,r);
Vd(1,k) = V2d(T(k), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I;
F(1,k) = V2d(T(k), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I;



%Iterate downwards to find threshold where optimal to wait
dum = 0;
for j = k-1:-1:k-u/dT
    Vd(1,j) = max (V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I, (lambda* Vd(1,k))/(lambda + r));
    if dum == 0 && Vd(1,j) > V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I
        dum = 1;
        T_s_d(1) = T(j);
    end  
    F(1,j) = V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I;
end

for j = k-u/dT-1:-1:1
    Vd(1,j) = max (V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I, (lambda* Vd(1,j+u/dT))/(lambda + r));
        if dum == 0 && Vd(1,j) > V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I
            dum = 1;
            T_s_d(1) = T(j);
        end  
    F(1,j) = V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(1),a)-I;
end

%Work upwards to Q10
Q1inf = n;
for i = 2:n
    dum = 0;
    P2e(i) = P2eOPT(v, Q20, gamma, eta2, P2i, Ti,  Q1(i), eta1, P1, a, Q10,r);
    pdi = exp(-Ti)*((Q1(i)-eta1*P1)/(Q10-eta1*P1))^a; %Is the total prob correct after follower has investes? Or shuold it be something else?
    Vd(i,k) = V2d(T(k), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I;
    F(i,k) = V2d(T(k), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I;
    for j = k-1:-1:k-u/dT
        F(i,j) = V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I;
        Vd(i,j) = max(V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I, OVd(P1, Q1(i), eta1, r, lambda, dQ1, Vd(i-1,j), Vd(i,k), a, Ti, Vm(i,j), Q10));
        if Vd(i,j) > V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I && dum ==0
            T_s_d(i) = T(j);
            dum = 1;
        end
    end 
    
    if abs(T_s_d(i) -(Tmax-2*dT)) <0.0000001 
        if Q1inf == 0;
            Q1inf = i-1;
        end
        F(i,j) = V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I;
        Vd(i,k) = OV2d (P1, Q1(i), eta1, r, dQ1, Vd(i-1,k), a, Ti, Vm(i,k), Q10); %Is it correct that this yields ish same value as OVd for large theta?
        for j = k-1:-1:k-u/dT
            Vd(i,j) = OVd(P1, Q1(i), eta1, r, lambda, dQ1, Vd(i-1,j), Vd(i,k), a, Ti, Vm(i,j), Q10);
        end
    end
    
    for j = k-u/dT-1:-1:1
        F(i,j) = V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I;
        Vd(i,j) = max(V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I, OVd(P1, Q1(i), eta1, r, lambda, dQ1, Vd(i-1,j), Vd(i,j+u/dT), a, Ti, Vm(i,j), Q10));
        if Vd(i,j) >  V2d(T(j), eta2, r, Q20, pdi,P2i, gamma,v, P2e(i),a)-I && dum == 0
            dum = 1;
           T_s_d (i) = T(j);
        end
    end
end

%Find where it goes down from infinity again
dum = 0;
Q1inf2 = n; 
for i = 1:n
    if abs(T_s_d(i) -(Tmax-2*dT)) <0.000001 
        dum = 1;
    end
    if dum == 1 && abs(T_s_d(i) -(Tmax-2*dT)) >0.0000001 && Q1inf2 ==n
        Q1inf2 = i;
    end
end

if Q1inf == 0 || Q1inf == 1 || Q1inf == 2 
    for i = 1:n
        if abs(T_s_d(i) -(Tmax-2*dT)) >0.0000001 && Q1inf2 ==n
            Q1inf2 = i;
        end
    end
end




