%Model inputs 
Q20 = 180; 
P1 = 3; 
eta1 = 20; 
eta2 = 15; 
I = 200;
r = 0.05; 

%Probability inputs
lambda = 2;
u = 0.2; 

%Grid size inputs
n = 1500; 
k = 500; 

%end values on grid
Tmax = 7; 
Q1max = 75; 

%Calculate step length
dQ1 = (Q1max-eta1*P1)/n ;
dT = Tmax/k ;
dum = round(u/dT);
dT = u/dum;
Tmax = k*dT;

%Set up grid
Q1 = zeros (1,n);
T = zeros (1,k);
Q1(1) = eta1*P1;
for i = 2:length(Q1)
    Q1(i) = Q1(i-1) + dQ1;
end
T(1) = 0;
for i = 2: length (T)
    T(i) = T(i-1) + dT;
end

%Set up arrays for investment threshold and firm value
T_s_m = ones(1,n)*(Tmax-2*dT);
Vm = zeros(n,k);
Vm(1, k) =   V2m(Tmax, eta2, r, Q20)-I;

dum = 0; % Dummy for checking whether shift from investment to wait has happened for given Q
Q1Inf = n; % index of the Q for which the threshold approach infinity

%Set up boundary condition at Q = eta1*P1
for j = k-1 : -1: k - u/dT %For loop to when j + u/dT cannot be reached
    Vm(1, j) = max((V2m(T(j), eta2, r, Q20)-I), (lambda* Vm(1,k))/(lambda + r) ); %At thetamax, the firm invests immediately.
    if (Vm(1 , j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0
         T_s_m(1) = T(j);
         dum = 1;
    end
end

for j = k- u/dT -1 : -1: 1 %For loop to when j + u/dT can be reached
    Vm(1, j) = max((V2m(T(j), eta2, r, Q20)-I), (lambda* Vm(1,j + u/dT))/(lambda + r) );
    if (Vm(1 , j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0
         T_s_m(1) = T(j);
         dum = 1;
    end
end

%Calculate forwards from Q=eta1*P1 + dQ
for i = 2 :n
    Vm(i, k) =   V2m(Tmax, eta2, r, Q20)-I; %At thetamax, the firm invests immediately.
    dum = 0;
    for j = k-1: -1 : k-u/dT %For loop to when j + u/dT cannot be reached
                Vm(i, j) = max(V2m(T(j), eta2, r, Q20)-I,  OVm(P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,k)));
        if (Vm(i, j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0 && dum == 0 
            dum = 1;
            T_s_m(i) = T(j);
        end
    end
        
    
    for j = k-u/dT-1 : -1 : 1  %For loop to when j + u/dT can be reached
        Vm(i, j) = max(V2m(T(j), eta2, r, Q20)-I,  OVm (P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,j+ u/dT)));
        if (Vm(i, j) > V2m(T(j), eta2, r, Q20)-I) && dum == 0
            dum = 1;
            T_s_m(i) = T(j);
        end
    end
    
    if abs(T_s_m(i)- (Tmax-2*dT)) < 0.00001 % If the threshold is Tmax, then it is not optimal to invest for Tmax for Q+dQ. Hence for loop is breaked
        Q1Inf = i-1; %Index for which Q(i-1) the threshold goes to infinity, and it is not optimal to invest at Tmax
        break;
    end
end

for i = Q1Inf: n
    Vm(i,k) = OV2m (P1, Q1(i), eta1, dQ1, Vm(i-1,k), r); %At Tmax, the added value from a jump can be ignored, due to exponential expression. 
    for j = k-1: -1: k-u/dT %For loop to when j + u/dT cannot be reached
        Vm(i,j) = OVm(P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,k));
    end
    for j = k-u/dT - 1: -1: 1 %For loop to when j + u/dT can be reached
        Vm(i,j) = OVm(P1, Q1(i), eta1, r, lambda, dQ1, Vm(i-1,j), Vm(i,j+u/dT));
    end
end

